@@ -1,8 +1,12 @@
-__version__ = "0.6.2"
+__version__ = "0.9.1"
 
 import warnings
 
-from .adaption import DomainAdaptionDataModule, PretrainingAdaptionDataModule
+from .adaption import (
+    DomainAdaptionDataModule,
+    PretrainingAdaptionDataModule,
+    LatentAlignDataModule,
+)
 from .baseline import BaselineDataModule, PretrainingBaselineDataModule
 from .core import RulDataModule
 from .reader import CmapssReader, FemtoReader, XjtuSyReader
