@@ -206,7 +206,7 @@ class CmapssReader(AbstractReader):
         return f"{split}_FD{self.fd:03d}.txt"
 
     def load_complete_split(
-        self, split: str
+        self, split: str, alias: str
     ) -> Tuple[List[np.ndarray], List[np.ndarray]]:
         file_path = self._get_feature_path(split)
         features, operation_conditions = self._load_features(file_path)
@@ -215,13 +215,20 @@ class CmapssReader(AbstractReader):
 
         if split in ["dev", "val"]:
             targets = self._generate_targets(time_steps)
-            features, targets = self._window_data(features, targets)
         elif split == "test":
-            targets = self._load_targets()
-            features = self._crop_data(features)
+            targets = self._load_targets(features)
         else:
             raise ValueError(f"Unknown split {split}.")
 
+        features, targets = self._pad_data(features, targets)
+
+        if alias in ["dev", "val"]:
+            features, targets = self._window_data(features, targets)
+        elif alias == "test":
+            features, targets = self._crop_data(features, targets)
+        else:
+            raise ValueError(f"Unknown alias {alias}.")
+
         return features, targets
 
     def _load_features(
@@ -281,44 +288,63 @@ class CmapssReader(AbstractReader):
 
         return targets
 
-    def _load_targets(self) -> List[np.ndarray]:
+    def _load_targets(self, features: List[np.ndarray]) -> List[np.ndarray]:
         """Load target file."""
         file_name = f"RUL_FD{self.fd:03d}.txt"
         file_path = os.path.join(self._CMAPSS_ROOT, file_name)
         raw_targets = np.loadtxt(file_path)
 
-        max_rul = self.max_rul or np.inf
-        raw_targets = np.minimum(max_rul, raw_targets)
         targets = np.split(raw_targets, len(raw_targets))
+        targets = [np.arange(len(f), 0, -1) + t - 1 for f, t in zip(features, targets)]
+
+        max_rul = self.max_rul or np.inf
+        targets = [np.minimum(max_rul, t) for t in targets]
 
         return targets
 
+    def _pad_data(self, features, targets):
+        padded_features = []
+        padded_targets = []
+        for i, (feat, target) in enumerate(zip(features, targets)):
+            if feat.shape[0] < self.window_size:
+                warnings.warn(
+                    f"Run {i} of CMAPSS FD{self.fd:03d} is shorter than window "
+                    f"size {self.window_size} and will be zero-padded."
+                )
+                missing = self.window_size - feat.shape[0]
+                feat_pad = (missing, feat.shape[1])
+                feat = np.concatenate([np.zeros(feat_pad), feat])
+                target = np.concatenate([np.zeros(missing), target])
+            padded_features.append(feat)
+            padded_targets.append(target)
+
+        return padded_features, padded_targets
+
     def _window_data(
         self, features: List[np.ndarray], targets: List[np.ndarray]
     ) -> Tuple[List[np.ndarray], List[np.ndarray]]:
         """Window features with specified window size."""
-        new_features = []
-        new_targets = []
+        windowed_features = []
+        windowed_targets = []
         for seq, target in zip(features, targets):
             windows = utils.extract_windows(seq, self.window_size)
             target = target[self.window_size - 1 :]
-            new_features.append(windows)
-            new_targets.append(target)
+            windowed_features.append(windows)
+            windowed_targets.append(target)
 
-        return new_features, new_targets
+        return windowed_features, windowed_targets
 
-    def _crop_data(self, features: List[np.ndarray]) -> List[np.ndarray]:
-        """Crop length of features to specified window size."""
+    def _crop_data(
+        self, features: List[np.ndarray], targets: List[np.ndarray]
+    ) -> Tuple[List[np.ndarray], List[np.ndarray]]:
+        """Crop length of data to specified window size."""
         cropped_features = []
-        for seq in features:
-            if seq.shape[0] < self.window_size:
-                pad = (self.window_size - seq.shape[0], seq.shape[1])
-                seq = np.concatenate([np.zeros(pad), seq])
-            else:
-                seq = seq[-self.window_size :]
-            cropped_features.append(np.expand_dims(seq, axis=0))
+        cropped_targets = []
+        for seq, target in zip(features, targets):
+            cropped_features.append(seq[None, -self.window_size :])
+            cropped_targets.append(target[-1, None])
 
-        return cropped_features
+        return cropped_features, cropped_targets
 
 
 def _download_cmapss(data_root: str) -> None:
